#!/usr/bin/perl -w
#
# bb-hphealth.plx
# ---------------
#
# checks various health conditions on hp servers
# and then feeds them to Big Brother.
#
# by Nick Nauwelaerts
# version 0.4 : 20090105
#		make it work with xymon
# version 0.3 : 20080811
#		also complain if no results are returned
# version 0.2 : 20070521
#		ignore nosuchobject errors (needed for nic monitoring)
# version 0.1 : 20060215


use strict;
use Net::SNMP;
use POSIX qw(strftime);

# variables used by the program, don't touch
my ($bbdisplay);	# server to which to send the big brother data
my ($bberror);		# used to determine which color the big brother test will be
my ($bbprog);		# program to call which sends data back to big brother
my ($bbstring);		# string to send to big brother
my ($bbtest);		# the name of the test for big brother
my ($community);	# snmp community string
my ($error);		# snmp connection error string
my ($host);		# host to check
my ($hostsfile);	# file which keeps host entries
my ($oidfile);		# file which keeps oid mappings
my ($session);		# snmp session
my ($time);		# timestamp
my ($tmpstring);	# string which will be fed to bbstring


# change these to what you need
$bbdisplay	= "bbdisplay.example.com";
$bbprog		= "/home/xymon/client/bin/bb";
$bbtest		= "health";
$community	= "public";
$hostsfile	= "/home/xymon/client/etc/bb-hphealth-hosts";
$oidfile	= "/home/xymon/client/etc/bb-hphealth-oid";


# test to see if the big brother binary is executable
if (!(-x $bbprog)) {
	warn ("the big brother binary $bbprog is not executable\n");
	exit (0);
}


open (HOSTS, $hostsfile) || die "couldn't open hostsfile: $!\n";
open (OIDS, $oidfile) || die "couldn't open oidfile: $!\n";


while (<HOSTS>) {
	if (!($_ =~ /^#/ ) && !($_ =~ /^\s+$/)) {
		$_ =~ s/\s+//g;
		$host = $_;
		($session, $error) = Net::SNMP->session(-hostname => $host, -timeout => 5,
			-community => $community, -port => 161, -version => 2);

		$bberror = 0;
		$time = (strftime "%A %B %e %H:%M:%S %Y", localtime);
		$tmpstring = "";
		seek (OIDS, 0, 0);

		# if there is an error with the snmp session just skip to the next host
		if (!($error =~ /^$/)) {
			print "$0: $error\n";
			next;
		}

		while (<OIDS>) {
			if (!($_ =~ /^#/ ) && !($_ =~ /^\s+$/)) {
				my $bad;
				my $good;
				my $ignore;
				my $name;
				my $oid;
				my $result;
				my $warn;
				
				chomp $_;
				($oid, $ignore, $good, $warn, $bad, $name) = split (/\s+/, $_, 6);

				$result = $session->get_request(-varbindlist => [$oid]);

				# break out of the loop if we detect an error
				$_ =  $session->error();
				if (($_ =~ /\w+/) && !($_ =~ /noSuchName/) && !($_ =~ /genError/)) {
					$tmpstring .= "<tr><td>an error occured in the snmp session:\n$_\n</td></tr>";
					$bberror = 2;
					last();
				}

				if (defined($result) || ($_ =~ /noSuchName/) || ($_ =~ /genError/)) {
					if (($_ =~ /noSuchName/) || ($_ =~ /noSuchObject/)) {
						# $result might be undefined, so check for this first
					} elsif (!($result->{$oid} =~ /(noSuchInstance|noSuchObject)/) && !($result->{$oid} eq $ignore)) {
						$tmpstring .= "<tr><td>$name</td>";
						if ($result->{$oid} == $good) {
							$tmpstring .= "<td>\&green normal</td>";
						} elsif ($result->{$oid} == $bad) {
							$tmpstring .= "<td>\&red alarm</td>";
							$bberror = 2;
						} elsif ($result->{$oid} == $warn) {
							$tmpstring .= "<td>\&yellow warning</td>";
							if ($bberror == 0){$bberror = 1};
						} else {
							$tmpstring .= "<td>\&red undefined returncode: $result->{$oid}</td>";
							$bberror = 2;
						}
						$tmpstring .= "</tr>\n";
					}
				} else {
					$tmpstring .= "<tr><td>$name</td>";
					$tmpstring .= "<tr><td>$oid</td>";
					$tmpstring .= "<td>\&red undefined error</td></tr>\n";
					$bberror = 2;
				}
			}
		}
		$session->close;

		# if there are no results, error out as well
		if ($tmpstring =~ /^$/) {
			$tmpstring = "<br><br>no results received";
			$bberror = 2;
		}

		$bbstring = "$bbprog $bbdisplay \'";
		$host =~ s/\./,/g;
		if ($bberror > 1) {
			$bbstring .= "status $host.$bbtest red ";
		} elsif ($bberror > 0) {
			$bbstring .= "status $host.$bbtest yellow ";
		} else {
			$bbstring .= "status $host.$bbtest green ";
		}
		$bbstring .= "$time" .
			" \n\n\<br><br><table width=\"100%\" border=\"1\"" .
			"cellspacing=\"1\" cellpadding=\"5\">";
		$bbstring .= $tmpstring;
		$bbstring .= "</table>\'\n";

		if ($bbstring ne "") {system ($bbstring);}
	}
}


close (HOSTS);
close (OIDS);
